@@ -89,14 +89,9 @@
 #' @param constant If \code{TRUE}, constant term is included in the model. Can
 #' also be a number (constant value). For \code{auto.msarima}, if \code{NULL},
 #' then the function will check if constant is needed.
-#' @param AR Vector or matrix of AR parameters. The order of parameters should
-#' be lag-wise. This means that first all the AR parameters of the firs lag
-#' should be passed, then for the second etc. AR of another \code{msarima()} can be
-#' passed here.
-#' @param MA Vector or matrix of MA parameters. The order of parameters should
-#' be lag-wise. This means that first all the MA parameters of the firs lag
-#' should be passed, then for the second etc. MA of another msarima can be
-#' passed here.
+#' @param arma Either the named list or a vector with AR / MA parameters ordered lag-wise.
+#' The number of elements should correspond to the specified orders e.g.
+#' \code{orders=list(ar=c(1,1),ma=c(1,1)), lags=c(1,4), arma=list(ar=c(0.9,0.8),ma=c(-0.3,0.3))}
 #' @param model Previously estimated MSARIMA model.
 #' @param initial Can be either character or a vector of initial states.
 #' If it is character, then it can be \code{"optimal"}, meaning that all initial
@@ -125,9 +120,7 @@
 #' \item \code{persistence} - the persistence vector. This is the place, where
 #' smoothing parameters live.
 #' \item \code{measurement} - measurement vector of the model.
-#' \item \code{AR} - the matrix of coefficients of AR terms.
-#' \item \code{I} - the matrix of coefficients of I terms.
-#' \item \code{MA} - the matrix of coefficients of MA terms.
+#' \item \code{arma} - list of AR/MA parameters
 #' \item \code{constant} - the value of the constant term.
 #' \item \code{initialType} - Type of the initial values used.
 #' \item \code{initial} - the initial values of the state vector (extracted
@@ -197,10 +190,11 @@
 #' @rdname msarima
 #' @export
 msarima <- function(y, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
-                    constant=FALSE, AR=NULL, MA=NULL, model=NULL,
-                    initial=c("backcasting","optimal","two-stage","complete"), ic=c("AICc","AIC","BIC","BICc"),
-                    loss=c("likelihood","MSE","MAE","HAM","MSEh","TMSE","GTMSE","MSCE"),
-                    h=10, holdout=FALSE,
+                    constant=FALSE, arma=NULL, model=NULL,
+                    initial=c("backcasting","optimal","two-stage","complete"),
+                    ic=c("AICc","AIC","BIC","BICc"),
+                    loss=c("likelihood","MSE","MAE","HAM","MSEh","TMSE","GTMSE","MSCE","GPL"),
+                    h=0, holdout=FALSE,
                     bounds=c("usual","admissible","none"),
                     silent=TRUE,
                     xreg=NULL, regressors=c("use","select","adapt"), initialX=NULL,
@@ -236,47 +230,13 @@ msarima <- function(y, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
 
 # If this is a normal ARIMA, do things
         if(any(unlist(gregexpr("combine",model$model))==-1)){
-            if(!is.null(model$occurrence)){
-                occurrence <- model$occurrence;
-            }
-            if(!is.null(model$initial)){
-                initial <- model$initial;
-            }
-            if(is.null(xreg)){
-                xreg <- model$xreg;
-            }
-            else{
-                if(is.null(model$xreg)){
-                    xreg <- NULL;
-                }
-                else{
-                    if(ncol(xreg)!=ncol(model$xreg)){
-                        xreg <- xreg[,colnames(model$xreg)];
-                    }
-                }
-            }
-            initialX <- model$initialX;
-            persistenceX <- model$persistenceX;
-            transitionX <- model$transitionX;
-            if(any(c(persistenceX)!=0) | any((transitionX!=0)&(transitionX!=1))){
-                updateX <- TRUE;
-            }
-            if(!is.null(model$AR)){
-                AR <- model$AR;
-            }
-            if(!is.null(model$MA)){
-                MA <- model$MA;
-            }
-            if(is.null(model$constant) || (is.numeric(model$constant) && model$constant==0)){
-                constant <- FALSE;
-            }
-            orders <- orders(model);
-            lags <- lags(model);
-            model <- model$model;
-            arimaOrders <- paste0(c("",substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1),"")
-                                   ,collapse=";");
-            comas <- unlist(gregexpr("\\,",arimaOrders));
-            semicolons <- unlist(gregexpr("\\;",arimaOrders));
+
+            return(adam(data=y, model=model,
+                        orders=orders, lags=lags, constant=constant,
+                        arma=arma,
+                        loss=loss, h=h, holdout=holdout, initial=initial,
+                        ic=ic, bounds=bounds[1], distribution="dnorm",
+                        silent=silent, regressors=regressors[1], environment=env, ...));
         }
         else{
             stop("The provided model is a combination of ARIMAs. We cannot fit that.",call.=FALSE);
@@ -350,7 +310,7 @@ msarima <- function(y, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
 
     ourModel <- adam(data=data, model="NNN",
                      orders=orders, lags=lags, constant=constant,
-                     arma=list(ar=AR,ma=MA),
+                     arma=arma,
                      loss=loss, h=h, holdout=holdout, initial=initialValue,
                      ic=ic, bounds=bounds[1], distribution="dnorm",
                      silent=silent, regressors=regressors[1], environment=env, ...);
