@@ -21,7 +21,7 @@
 auto.ces <- function(y, seasonality=c("none","simple","partial","full"), lags=c(frequency(y)),
                      initial=c("backcasting","optimal","two-stage","complete"),
                      ic=c("AICc","AIC","BIC","BICc"),
-                     loss=c("likelihood","MSE","MAE","HAM","MSEh","TMSE","GTMSE","MSCE"),
+                     loss=c("likelihood","MSE","MAE","HAM","MSEh","TMSE","GTMSE","MSCE","GPL"),
                      h=0, holdout=FALSE,
                      bounds=c("admissible","none"),
                      silent=TRUE, xreg=NULL, regressors=c("use","select","adapt"), ...){
