@@ -42,11 +42,12 @@
 #' \item \code{model} - Name of CES model.
 #' \item \code{a} - Value of complex smoothing parameter a. If \code{nsim>1}, then
 #' this is a vector.
-#' \item \code{b} - Value of complex smoothing parameter b. If \code{seasonality="n"}
-#' or \code{seasonality="s"}, then this is equal to NULL. If \code{nsim>1},
+#' \item \code{b} - Value of complex smoothing parameter b. If \code{seasonality="none"}
+#' or \code{seasonality="simple"}, then this is equal to NULL. If \code{nsim>1},
 #' then this is a vector.
 #' \item \code{initial} - Initial values of CES in a form of matrix. If \code{nsim>1},
 #' then this is an array.
+#' \item \code{profile} - The final profile produced in the simulation.
 #' \item \code{data} - Time series vector (or matrix if \code{nsim>1}) of the generated
 #' series.
 #' \item \code{states} - Matrix (or array if \code{nsim>1}) of states. States are in
@@ -80,17 +81,17 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
                     initial=NULL,
                     randomizer=c("rnorm","rt","rlaplace","rs"),
                     probability=1, ...){
-# Function simulates the data using CES state space framework
-#
-# seasonality - the type of seasonality to produce.
-# frequency - the frequency of the data. In the case of seasonal models must be > 1.
-# a, b - complex smoothing parameters.
-# initial - the vector of initial states,
-#    If NULL it will be generated.
-# obs - the number of observations in each time series.
-# nsim - the number of series needed to be generated.
-# randomizer - the type of the random number generator function
-# ... - the parameters passed to the randomizer.
+    # Function simulates the data using CES state space framework
+    #
+    # seasonality - the type of seasonality to produce.
+    # frequency - the frequency of the data. In the case of seasonal models must be > 1.
+    # a, b - complex smoothing parameters.
+    # initial - the vector of initial states,
+    #    If NULL it will be generated.
+    # obs - the number of observations in each time series.
+    # nsim - the number of series needed to be generated.
+    # randomizer - the type of the random number generator function
+    # ... - the parameters passed to the randomizer.
 
     randomizer <- randomizer[1];
 
@@ -114,15 +115,10 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
         return(aValue);
     }
 
-#### Check values and preset parameters ####
-# If the user typed wrong seasonality, use "none" instead
-    if(all(seasonality!=c("n","s","p","f","none","simple","partial","full"))){
-        warning(paste0("Wrong seasonality type: '",seasonality, "'. Changing to 'none'"), call.=FALSE);
-        seasonality <- "n";
-    }
-    seasonality <- substring(seasonality[1],1,1);
+    #### Check values and preset parameters ####
+    seasonality <- match.arg(seasonality);
 
-    if(seasonality!="n" & frequency==1){
+    if(seasonality!="none" & frequency==1){
         stop("Can't simulate seasonal data with frequency=1!",call.=FALSE)
     }
 
@@ -135,25 +131,25 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
     else{
         a$generate <- FALSE;
         if(!(((Re(a$value)-2.5)^2 + Im(a$value)^2 > 1.25) &
-                   ((Re(a$value)-0.5)^2 + (Im(a$value)-1)^2 > 0.25) &
-                   (Re(a$value)-1.5)^2 + (Im(a$value)-0.5)^2 < 1.5)){
+             ((Re(a$value)-0.5)^2 + (Im(a$value)-1)^2 > 0.25) &
+             (Re(a$value)-1.5)^2 + (Im(a$value)-0.5)^2 < 1.5)){
             warning("The provided complex smoothing parameter a leads to non-stable model!",call.=FALSE);
         }
     }
 
-    if(all(is.null(b$value),any(seasonality==c("p","f")))){
+    if(all(is.null(b$value),any(seasonality==c("partial","full")))){
         b$generate <- TRUE;
     }
     else{
         b$generate <- FALSE;
-        if(seasonality=="f"){
+        if(seasonality=="full"){
             if(!(((Re(b$value)-2.5)^2 + Im(b$value)^2 > 1.25) &
                  ((Re(b$value)-0.5)^2 + (Im(b$value)-1)^2 > 0.25) &
                  (Re(b$value)-1.5)^2 + (Im(b$value)-0.5)^2 < 1.5)){
                 warning("The provided complex smoothing parameter b leads to non-stable model!",call.=FALSE);
             }
         }
-        else if(seasonality=="p"){
+        else if(seasonality=="partial"){
             if((b$value<0) | (b$value>1)){
                 warning("Be careful with the provided b parameter - the model can be unstable.",call.=FALSE);
             }
@@ -161,8 +157,8 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
     }
 
     a$number <- 2;
-# Define lags, number of components and number of parameters
-    if(seasonality=="n"){
+    # Define lags, number of components and number of parameters
+    if(seasonality=="none"){
         # No seasonality
         lagsModelMax <- 1;
         lagsModel <- c(1,1);
@@ -170,38 +166,38 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
         componentsNumber <- 2;
         b$number <- 0;
         componentsNames <- c("level","potential");
-        matw <- matrix(c(1,0),1,2);
+        matWt <- matrix(c(1,0),obs,2,byrow=TRUE);
     }
-    else if(seasonality=="s"){
+    else if(seasonality=="simple"){
         # Simple seasonality, lagged CES
         lagsModelMax <- frequency;
         lagsModel <- c(lagsModelMax,lagsModelMax);
         componentsNumber <- 2;
         b$number <- 0;
         componentsNames <- c("seasonal level","seasonal potential");
-        matw <- matrix(c(1,0),1,2);
+        matWt <- matrix(c(1,0),obs,2,byrow=TRUE);
     }
-    else if(seasonality=="p"){
+    else if(seasonality=="partial"){
         # Partial seasonality with a real part only
         lagsModelMax <- frequency;
         lagsModel <- c(1,1,lagsModelMax);
         componentsNumber <- 3;
         b$number <- 1;
         componentsNames <- c("level","potential","seasonal");
-        matw <- matrix(c(1,0,1),1,3);
+        matWt <- matrix(c(1,0,1),obs,3,byrow=TRUE);
     }
-    else if(seasonality=="f"){
+    else if(seasonality=="full"){
         # Full seasonality with both real and imaginary parts
         lagsModelMax <- frequency;
         lagsModel <- c(1,1,lagsModelMax,lagsModelMax);
         componentsNumber <- 4;
         b$number <- 2;
         componentsNames <- c("level","potential","seasonal level","seasonal potential");
-        matw <- matrix(c(1,0,1,0),1,4);
+        matWt <- matrix(c(1,0,1,0),obs,4,byrow=TRUE);
     }
 
     initialValue <- initial;
-# Initial values
+    # Initial values
     if(!is.null(initialValue)){
         if(length(initialValue) != lagsModelMax*componentsNumber){
             warning(paste0("Wrong length of initial vector. Should be ",lagsModelMax*componentsNumber,
@@ -219,25 +215,25 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
         initialGenerate <- TRUE;
     }
 
-# In the case of wrong nsim, make it natural number. The same is for obs and frequency.
+    # In the case of wrong nsim, make it natural number. The same is for obs and frequency.
     nsim <- abs(round(nsim,0));
     obs <- abs(round(obs,0));
     obsStates <- obs + lagsModelMax;
     frequency <- abs(round(frequency,0));
 
-# Define arrays
-    arrvt <- array(NA,c(obsStates,componentsNumber,nsim),dimnames=list(NULL,componentsNames,NULL));
+    # Define arrays
+    arrVt <- array(NA,c(componentsNumber,obsStates,nsim),dimnames=list(componentsNames,NULL,NULL));
     arrF <- array(0,c(componentsNumber,componentsNumber,nsim));
-    matg <- matrix(0,componentsNumber,nsim);
+    matG <- matrix(0,componentsNumber,nsim);
 
-    materrors <- matrix(NA,obs,nsim);
-    matyt <- matrix(NA,obs,nsim);
-    matot <- matrix(NA,obs,nsim);
-    matInitialValue <- array(NA,c(lagsModelMax,componentsNumber,nsim));
+    matErrors <- matrix(NA,obs,nsim);
+    matYt <- matrix(NA,obs,nsim);
+    matOt <- matrix(NA,obs,nsim);
+    matInitialValue <- array(NA,c(componentsNumber,lagsModelMax,nsim));
     aValue <- matrix(NA,2,nsim);
     bValue <- matrix(NA,b$number,nsim);
 
-# Check the vector of probabilities
+    # Check the vector of probabilities
     if(is.vector(probability)){
         if(any(probability!=probability[1])){
             if(length(probability)!=obs){
@@ -254,20 +250,20 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
         }
     }
 
-#### Generate stuff if needed ####
-# First deal with initials
+    #### Generate stuff if needed ####
+    # First deal with initials
     if(initialGenerate){
         matInitialValue[,,] <- runif(componentsNumber*nsim*lagsModelMax,0,1000);
-        if(all(seasonality!=c("n","s"))){
-            matInitialValue[1:lagsModelMax,1:2,] <- rep(matInitialValue[lagsModelMax,1:2,],each=lagsModelMax);
+        if(all(seasonality!=c("none","simple"))){
+            matInitialValue[1:2,1:lagsModelMax,] <- rep(matInitialValue[1:2,lagsModelMax,],each=lagsModelMax);
         }
     }
     else{
-        matInitialValue[1:lagsModelMax,,] <- rep(initialValue,nsim);
+        matInitialValue[,1:lagsModelMax,] <- rep(initialValue,each=nsim);
     }
-    arrvt[1:lagsModelMax,,] <- matInitialValue;
+    arrVt[1:componentsNumber,1:lagsModelMax,] <- matInitialValue;
 
-# Now let's do parameters with transition + persistence
+    # Now let's do parameters with transition + persistence
     if(a$generate){
         aValue[,] <- AGenerator(nsim);
     }
@@ -278,7 +274,7 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
 
     if(b$number!=0){
         if(b$generate){
-            if(seasonality=="f"){
+            if(seasonality=="full"){
                 bValue[,] <- AGenerator(nsim);
             }
             else{
@@ -286,7 +282,7 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
             }
         }
         else{
-            if(seasonality=="f"){
+            if(seasonality=="full"){
                 bValue[1,] <- Re(b$value);
                 bValue[2,] <- Im(b$value);
             }
@@ -299,18 +295,18 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
     arrF[1:2,1,] <- 1;
     for(i in 1:nsim){
         arrF[1:2,2,i] <- c(aValue[2,i]-1,1-aValue[1,i]);
-        matg[1:2,i] <- c(aValue[1,i]-aValue[2,i],aValue[1,i]+aValue[2,i]);
+        matG[1:2,i] <- c(aValue[1,i]-aValue[2,i],aValue[1,i]+aValue[2,i]);
     }
 
-    if(seasonality=="p"){
+    if(seasonality=="partial"){
         arrF[3,3,] <- 1;
-        matg[3,] <- bValue[1,];
+        matG[3,] <- bValue[1,];
     }
-    else if(seasonality=="f"){
+    else if(seasonality=="full"){
         arrF[3:4,3,] <- 1;
         for(i in 1:nsim){
             arrF[3:4,4,i] <- c(bValue[2,i]-1,1-bValue[1,i]);
-            matg[3:4,i] <- c(bValue[1,i]-bValue[2,i],bValue[1,i]+bValue[2,i]);
+            matG[3:4,i] <- c(bValue[1,i]-bValue[2,i],bValue[1,i]+bValue[2,i]);
         }
     }
 
@@ -325,69 +321,116 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
         ellipsis$n <- nsim*obs;
         # Create vector of the errors
         if(any(randomizer==c("rnorm","rlaplace","rs"))){
-            materrors[,] <- do.call(randomizer,ellipsis);
+            matErrors[,] <- do.call(randomizer,ellipsis);
         }
         else if(randomizer=="rt"){
             # The degrees of freedom are df = n - k.
-            materrors[,] <- rt(nsim*obs,obs-(componentsNumber + lagsModelMax));
+            matErrors[,] <- rt(nsim*obs,obs-(componentsNumber + lagsModelMax));
         }
 
         # Center errors just in case
-        materrors <- materrors - colMeans(materrors);
+        matErrors <- matErrors - colMeans(matErrors);
         # Change variance to make some sense. Errors should not be rediculously high and not too low.
-        materrors <- materrors * sqrt(abs(colMeans(as.matrix(arrvt[1:lagsModelMax,1,]))));
+        matErrors <- matErrors * sqrt(abs(colMeans(as.matrix(arrVt[1,1:lagsModelMax,]))));
         if(randomizer=="rs"){
-            materrors <- materrors / 4;
+            matErrors <- matErrors / 4;
         }
     }
     # If arguments are passed, use them. WE ASSUME HERE THAT USER KNOWS WHAT HE'S DOING!
     else{
         ellipsis$n <- nsim*obs;
-        materrors[,] <- do.call(randomizer,ellipsis);
+        matErrors[,] <- do.call(randomizer,ellipsis);
         if(randomizer=="rbeta"){
             # Center the errors around 0
-            materrors <- materrors - 0.5;
+            matErrors <- matErrors - 0.5;
             # Make a meaningful variance of data. Something resembling to var=1.
-            materrors <- materrors / rep(sqrt(colMeans(materrors^2)) *
-                                             sqrt(abs(colMeans(as.matrix(arrvt[1:lagsModelMax,1,])))),each=obs);
+            matErrors <- matErrors / rep(sqrt(colMeans(matErrors^2)) *
+                                             sqrt(abs(colMeans(as.matrix(arrVt[1,1:lagsModelMax,])))),each=obs);
         }
         else if(randomizer=="rt"){
             # Make a meaningful variance of data.
-            materrors <- materrors * rep(sqrt(abs(colMeans(as.matrix(arrvt[1:lagsModelMax,1,])))),each=obs);
+            matErrors <- matErrors * rep(sqrt(abs(colMeans(as.matrix(arrVt[1,1:lagsModelMax,])))),each=obs);
         }
     }
 
-# Generate ones for the possible intermittency
+    # Generate ones for the possible intermittency
     if(all(probability == 1)){
-        matot[,] <- 1;
+        matOt[,] <- 1;
     }
     else{
-        matot[,] <- rbinom(obs*nsim,1,probability);
+        matOt[,] <- rbinom(obs*nsim,1,probability);
     }
 
-#### Simulate the data ####
-    simulateddata <- simulatorwrap(arrvt,materrors,matot,arrF,matw,matg,"A","N","N",lagsModel);
+    #### Variables for the adamCore ####
+    lagsModelSeasonal <- lagsModel[lagsModel>1];
+    nSeasonal <- length(lagsModelSeasonal);
+    xregNumber <- 0;
+    constantRequired <- FALSE;
+    adamETS <- FALSE;
+    # Create all the necessary matrices and vectors
+    componentsNumberARIMA <- componentsNumber <- switch(seasonality,
+                                                        "none"=2,
+                                                        "simple"=nSeasonal,
+                                                        "partial"=2+nSeasonal,
+                                                        "full"=2+nSeasonal);
+
+    componentsNumberETS <- componentsNumberETSSeasonal <- componentsNumberETSNonSeasonal <- 0;
+
+    lagsModelAll <- matrix(c(switch(seasonality,
+                                    "none"=c(1,1),
+                                    "simple"=lagsModelSeasonal,
+                                    "partial"=c(1,1,lagsModelSeasonal),
+                                    "full"=c(1,1,lagsModelSeasonal)),
+                             rep(1, xregNumber)),
+                           ncol=1);
+
+    Etype <- "A";
+    Stype <- Ttype <- "N";
+
+    profiles <- adamProfileCreator(lagsModelAll, max(lagsModelAll), obs);
+    indexLookupTable <- profiles$lookup;
+    profilesRecentArray <- array(matInitialValue,
+                                 c(componentsNumberETS+componentsNumberARIMA+xregNumber+constantRequired,
+                                   lagsModelMax,
+                                   nsim));
+
+    # Create C++ adam class, which will then use fit, forecast etc methods
+    adamCpp <- new(adamCore,
+                   lagsModelAll, Etype, Ttype, Stype,
+                   componentsNumberETSNonSeasonal,
+                   componentsNumberETSSeasonal,
+                   componentsNumberETS, componentsNumberARIMA,
+                   xregNumber, length(lagsModelAll),
+                   constantRequired, adamETS);
+
+    #### Simulate the data ####
+    simulateddata <- adamCpp$simulate(matErrors, matOt,
+                                      arrVt, matWt,
+                                      arrF,
+                                      matG,
+                                      indexLookupTable, profilesRecentArray,
+                                      Etype);
 
     if(all(probability == 1)){
-        matyt <- simulateddata$matyt;
+        matYt <- simulateddata$data;
     }
     else{
-        matyt <- round(simulateddata$matyt,0);
+        matYt <- round(simulateddata$data,0);
     }
-    arrvt <- simulateddata$arrvt;
-    dimnames(arrvt) <- list(NULL,componentsNames,NULL);
+    arrVt[] <- simulateddata$states;
+    profilesRecentArray[] <- simulateddata$profile;
 
     if(any(randomizer==c("rnorm","rt"))){
-        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(materrors^2)));
+        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(matErrors^2)));
     }
     else if(randomizer=="rlaplace"){
-        veclikelihood <- -obs*(log(2*exp(1)) + log(colMeans(abs(materrors))));
+        veclikelihood <- -obs*(log(2*exp(1)) + log(colMeans(abs(matErrors))));
     }
     else if(randomizer=="rs"){
-        veclikelihood <- -2*obs*(log(2*exp(1)) + log(0.5*colMeans(sqrt(abs(materrors)))));
+        veclikelihood <- -2*obs*(log(2*exp(1)) + log(0.5*colMeans(sqrt(abs(matErrors)))));
     }
     else if(randomizer=="rlnorm"){
-        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(materrors^2))) - colSums(log(matyt));
+        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(matErrors^2))) - colSums(log(matYt));
     }
     # If this is something unknown, forget about it
     else{
@@ -395,16 +438,16 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
     }
 
     if(nsim==1){
-        matyt <- ts(matyt[,1],frequency=frequency);
-        materrors <- ts(materrors[,1],frequency=frequency);
-        arrvt <- ts(arrvt[,,1],frequency=frequency,start=c(0,frequency-lagsModelMax+1));
-        matot <- ts(matot[,1],frequency=frequency);
+        matYt <- ts(matYt[,1],frequency=frequency);
+        matErrors <- ts(matErrors[,1],frequency=frequency);
+        arrVt <- ts(arrVt[,,1],frequency=frequency,start=c(0,frequency-lagsModelMax+1));
+        matOt <- ts(matOt[,1],frequency=frequency);
         matInitialValue <- matInitialValue[,,1];
     }
     else{
-        matyt <- ts(matyt,frequency=frequency);
-        materrors <- ts(materrors,frequency=frequency);
-        matot <- ts(matot,frequency=frequency);
+        matYt <- ts(matYt,frequency=frequency);
+        matErrors <- ts(matErrors,frequency=frequency);
+        matOt <- ts(matOt,frequency=frequency);
     }
 
     modelname <- paste0("CES(",seasonality,")");
@@ -413,16 +456,16 @@ sim.ces <- function(seasonality=c("none","simple","partial","full"),
     }
 
     aValue <- complex(real=aValue[1,],imaginary=aValue[2,]);
-    if(any(seasonality==c("n","s"))){
+    if(any(seasonality==c("none","simple"))){
         bValue <- NULL;
     }
-    else if(seasonality=="f"){
+    else if(seasonality=="full"){
         bValue <- complex(real=bValue[1,],imaginary=bValue[2,]);
     }
 
-    model <- list(model=modelname,
+    model <- list(model=modelname, profile=profilesRecentArray,
                   a=aValue, b=bValue, initial=matInitialValue,
-                  data=matyt, states=arrvt, residuals=materrors,
-                  occurrence=matot, logLik=veclikelihood);
+                  data=matYt, states=arrVt, residuals=matErrors,
+                  occurrence=matOt, logLik=veclikelihood);
     return(structure(model,class="smooth.sim"));
 }
