@@ -55,6 +55,7 @@
 #' \item \code{phi} - Value of damping parameter used in time series generation.
 #' \item \code{initial} - Vector (or matrix) of initial values.
 #' \item \code{initialSeason} - Vector (or matrix) of initial seasonal coefficients.
+#' \item \code{profile} - The final profile produced in the simulation.
 #' \item \code{probability} - vector of probabilities used in the simulation.
 #' \item \code{intermittent} - type of the intermittent model used.
 #' \item \code{residuals} - Error terms used in the simulation. Either vector or matrix,
@@ -89,7 +90,7 @@
 #'
 #' # Create 60 observations of monthly data using MAM model
 #' # with errors from uniform distribution
-#' ETSMAM <- sim.es(model="MAM",persistence=c(0.3,0.2,0.1),initial=c(2000,50),
+#' ETSMAM <- sim.es(model="MAdM",persistence=c(0.3,0.2,0.1),initial=c(2000,50),
 #'            phi=0.8,frequency=12,obs=60,randomizer="runif",min=-0.5,max=0.5)
 #'
 #' # Create 80 observations of quarterly data using MMM model
@@ -115,27 +116,19 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
                    bounds=c("usual","admissible","restricted"),
                    randomizer=c("rnorm","rlnorm","rt","rlaplace","rs"),
                    probability=1, ...){
-# Function generates data using ETS with Single Source of Error as a data generating process.
-#    Copyright (C) 2015 - Inf Ivan Svetunkov
+    # Function generates data using ETS with Single Source of Error as a data generating process.
+    #    Copyright (C) 2015 - Inf Ivan Svetunkov
 
     randomizer <- randomizer[1];
     ellipsis <- list(...);
-    bounds <- bounds[1];
+    bounds <- match.arg(bounds);
     # If R decided that by "b" we meant "bounds", fix this!
     if(is.numeric(bounds)){
         ellipsis$b <- bounds;
-        bounds <- "u";
+        bounds <- "usual";
     }
 
-    if(all(bounds!=c("u","a","r","usual","admissible","restricted"))){
-        warning(paste0("Strange type of bounds provided: ",bounds,". Switching to 'usual'."),
-                call.=FALSE);
-        bounds <- "u";
-    }
-
-    bounds <- substring(bounds[1],1,1);
-
-# If chosen model is "AAdN" or anything like that, we are taking the appropriate values
+    # If chosen model is "AAdN" or anything like that, we are taking the appropriate values
     if(nchar(model)==4){
         Etype <- substring(model,1,1);
         Ttype <- substring(model,2,2);
@@ -165,7 +158,7 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         stop(paste0("You have defined a strange model: ",model,". Cannot proceed"),call.=FALSE);
     }
 
-# In the case of wrong nsim, make it natural number. The same is for obs and frequency.
+    # In the case of wrong nsim, make it natural number. The same is for obs and frequency.
     nsim <- abs(round(nsim,0));
     obs <- abs(round(obs,0));
     frequency <- abs(round(frequency,0));
@@ -179,25 +172,25 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
                        ". Be careful!"),call.=FALSE);
     }
 
-# Check the used model and estimate the length of needed persistence vector.
+    # Check the used model and estimate the length of needed persistence vector.
     if(Etype!="A" & Etype!="M"){
         stop("Wrong error type! Should be 'A' or 'M'.",call.=FALSE);
     }
     else{
-# The number of the smoothing parameters needed
+        # The number of the smoothing parameters needed
         persistenceLength <- 1;
-# The number initial values of the state vector
+        # The number initial values of the state vector
         componentsNumber <- 1;
-# The lag of components (needed for the seasonal models)
+        # The lag of components (needed for the seasonal models)
         lagsModel <- 1;
-# The names of the state vector components
+        # The names of the state vector components
         componentsNames <- "level";
-        matw <- 1;
-# The transition matrix
+        matWt <- 1;
+        # The transition matrix
         matF <- matrix(1,1,1);
     }
 
-# Check the trend type of the model
+    # Check the trend type of the model
     if(Ttype!="N" & Ttype!="A" & Ttype!="M"){
         stop("Wrong trend type! Should be 'N', 'A' or 'M'.",call.=FALSE);
     }
@@ -209,7 +202,7 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         componentsNumber <- componentsNumber + 1;
         lagsModel <- c(lagsModel,1);
         componentsNames <- c(componentsNames,"trend");
-        matw <- c(matw,phi);
+        matWt <- c(matWt,phi);
         matF <- matrix(c(1,0,phi,phi),2,2);
         componentTrend=TRUE;
         if(phi!=1){
@@ -220,7 +213,7 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         componentTrend=FALSE;
     }
 
-# Check the seasonaity type of the model
+    # Check the seasonaity type of the model
     if(Stype!="N" & Stype!="A" & Stype!="M"){
         stop("Wrong seasonality type! Should be 'N', 'A' or 'M'.",call.=FALSE);
     }
@@ -231,11 +224,11 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
 
     if(Stype!="N"){
         persistenceLength <- persistenceLength + 1;
-# lagsModelMax is used in the cases of seasonal models.
-#   if lagsModelMax==1 then non-seasonal data will be produced with the defined frequency.
+        # lagsModelMax is used in the cases of seasonal models.
+        #   if lagsModelMax==1 then non-seasonal data will be produced with the defined frequency.
         lagsModel <- c(lagsModel,frequency);
         componentsNames <- c(componentsNames,"seasonality");
-        matw <- c(matw,1);
+        matWt <- c(matWt,1);
         componentSeasonal <- TRUE;
 
         if(!componentTrend){
@@ -249,13 +242,13 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         componentSeasonal <- FALSE;
     }
 
-# Make matrices
+    # Make matrices
     lagsModel <- matrix(lagsModel,persistenceLength,1);
     lagsModelMax <- max(lagsModel);
-    matw <- matrix(matw,1,persistenceLength);
+    matWt <- matrix(matWt,obs,persistenceLength, byrow=TRUE);
     arrF <- array(matF,c(dim(matF),nsim));
 
-# Check the persistence vector length
+    # Check the persistence vector length
     if(!is.null(persistence)){
         if(persistenceLength != length(persistence)){
             if(length(persistence)!=1){
@@ -269,7 +262,7 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         }
     }
 
-# Check the inital vector length
+    # Check the inital vector length
     if(!is.null(initial)){
         if(length(initial)>2){
             stop("The length of the initial value is wrong! It should not be greater than 2.",call.=FALSE);
@@ -288,7 +281,7 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         }
     }
 
-# Check the inital seasonal vector length
+    # Check the inital seasonal vector length
     if(!is.null(initialSeason)){
         if(lagsModelMax!=length(initialSeason)){
             warning(paste0("The length of seasonal initial states does not correspond to the chosen frequency!\n",
@@ -303,7 +296,7 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         randomizer = "rnorm";
     }
 
-# Check the vector of probabilities
+    # Check the vector of probabilities
     if(is.vector(probability)){
         if(any(probability!=probability[1])){
             if(length(probability)!=obs){
@@ -323,7 +316,7 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         }
     }
 
-# Check the probabilities and try to assign the type of intermittent model
+    # Check the probabilities and try to assign the type of intermittent model
     if(length(probability)==1){
         intermittent <- "fixed";
     }
@@ -336,138 +329,143 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         intermittent <- "none";
     }
 
-##### Let's make sum fun #####
-    matg <- matrix(NA,persistenceLength,nsim);
-    arrvt <- array(NA,c(obs+lagsModelMax,persistenceLength,nsim),dimnames=list(NULL,componentsNames,NULL));
-    materrors <- matrix(NA,obs,nsim);
-    matyt <- matrix(NA,obs,nsim);
-    matot <- matrix(NA,obs,nsim);
+    ##### Let's make sum fun #####
+    matG <- matrix(NA,persistenceLength,nsim);
+    arrVt <- array(NA,c(persistenceLength,obs+lagsModelMax,nsim),dimnames=list(componentsNames,NULL,NULL));
+    matErrors <- matrix(NA,obs,nsim);
+    matYt <- matrix(NA,obs,nsim);
+    matOt <- matrix(NA,obs,nsim);
 
-# If the persistence is NULL or was of the wrong length, generate the values
+    # If the persistence is NULL or was of the wrong length, generate the values
     if(is.null(persistence)){
-### For the case of "usual" bounds make restrictions on the generated smoothing parameters so the ETS can be "averaging" model.
+        ### For the case of "usual" bounds make restrictions on the generated smoothing parameters so the ETS can be "averaging" model.
 
-### First generate the first smoothing parameter.
-        if(bounds=="u"){
-            matg[1,] <- runif(nsim,0,1);
+        ### First generate the first smoothing parameter.
+        if(bounds=="usual"){
+            matG[1,] <- runif(nsim,0,1);
         }
-### These restrictions are even touhger
-        else if(bounds=="r"){
-            matg[1,] <- runif(nsim,0,0.3);
+        ### These restrictions are even touhger
+        else if(bounds=="restricted"){
+            matG[1,] <- runif(nsim,0,0.3);
         }
 
-### Fill in the other smoothing parameters
-        if(bounds!="a"){
+        ### Fill in the other smoothing parameters
+        if(bounds!="admissible"){
             if(Ttype!="N"){
-                matg[2,] <- runif(nsim,0,matg[1,]);
+                matG[2,] <- runif(nsim,0,matG[1,]);
             }
             if(Stype!="N"){
-                matg[persistenceLength,] <- runif(nsim,0,max(0,1-matg[1]));
+                matG[persistenceLength,] <- runif(nsim,0,max(0,1-matG[1]));
             }
         }
-### In case of admissible bounds, do some stuff
+        ### In case of admissible bounds, do some stuff
         else{
-            matg[,] <- runif(persistenceLength*nsim,1-1/phi,1+1/phi);
+            matG[,] <- runif(persistenceLength*nsim,1-1/phi,1+1/phi);
             if(Ttype!="N"){
-                matg[2,] <- runif(nsim,matg[1,]*(phi-1),(2-matg[1,])*(1+phi));
+                matG[2,] <- runif(nsim,matG[1,]*(phi-1),(2-matG[1,])*(1+phi));
                 if(Stype!="N"){
                     Theta.func <- function(Theta){
-                        result <- (phi*matg[1,i]+phi+1)/(matg[3,i]) +
+                        result <- (phi*matG[1,i]+phi+1)/(matG[3,i]) +
                             ((phi-1)*(1+cos(Theta)-cos(lagsModelMax*Theta)) +
                                  cos((lagsModelMax-1)*Theta)-phi*cos((lagsModelMax+1)*Theta))/(2*(1+cos(Theta))*(1-cos(lagsModelMax*Theta)));
                         return(abs(result));
                     }
 
                     for(i in 1:nsim){
-                        matg[3,i] <- runif(1,max(1-1/phi-matg[1,i],0),1+1/phi-matg[1,i]);
+                        matG[3,i] <- runif(1,max(1-1/phi-matG[1,i],0),1+1/phi-matG[1,i]);
 
-                        B <- phi*(4-3*matg[3,i])+matg[3,i]*(1-phi)/lagsModelMax;
-                        C <- sqrt(B^2-8*(phi^2*(1-matg[3,i])^2+2*(phi-1)*(1-matg[3,i])-1)+8*matg[3,i]^2*(1-phi)/lagsModelMax);
-                        matg[1,i] <- runif(1,1-1/phi-matg[3,i]*(1-lagsModelMax+phi*(1+lagsModelMax))/(2*phi*lagsModelMax),(B+C)/(4*phi));
-# Solve the equation to get Theta value. Theta
+                        B <- phi*(4-3*matG[3,i])+matG[3,i]*(1-phi)/lagsModelMax;
+                        C <- sqrt(B^2-8*(phi^2*(1-matG[3,i])^2+2*(phi-1)*(1-matG[3,i])-1)+8*matG[3,i]^2*(1-phi)/lagsModelMax);
+                        matG[1,i] <- runif(1,1-1/phi-matG[3,i]*(1-lagsModelMax+phi*(1+lagsModelMax))/(2*phi*lagsModelMax),(B+C)/(4*phi));
+                        # Solve the equation to get Theta value. Theta
 
                         Theta <- 0.1;
                         Theta <- optim(Theta,Theta.func,method="Brent",lower=0,upper=1)$par;
 
-                        D <- (phi*(1-matg[1,i])+1)*(1-cos(Theta)) - matg[3,i]*((1+phi)*(1-cos(Theta) - cos(lagsModelMax*Theta)) +
+                        D <- (phi*(1-matG[1,i])+1)*(1-cos(Theta)) - matG[3,i]*((1+phi)*(1-cos(Theta) - cos(lagsModelMax*Theta)) +
                                                                                    cos((lagsModelMax-1)*Theta)+phi*cos((lagsModelMax+1)*Theta))/
                             (2*(1+cos(Theta))*(1-cos(lagsModelMax*Theta)));
-                        matg[2,i] <- runif(1,-(1-phi)*(matg[3,i]/lagsModelMax+matg[1,i]),D+(phi-1)*matg[1,i]);
+                        matG[2,i] <- runif(1,-(1-phi)*(matG[3,i]/lagsModelMax+matG[1,i]),D+(phi-1)*matG[1,i]);
                     }
                 }
             }
             else{
                 if(Stype!="N"){
-                    matg[1,] <- runif(nsim,-2/(lagsModelMax-1),2);
+                    matG[1,] <- runif(nsim,-2/(lagsModelMax-1),2);
                     for(i in 1:nsim){
-                        matg[2,i] <- runif(1,max(-lagsModelMax*matg[1,i],0),2-matg[1,i]);
+                        matG[2,i] <- runif(1,max(-lagsModelMax*matG[1,i],0),2-matG[1,i]);
                     }
-                    matg[1,] <- runif(nsim,-2/(lagsModelMax-1),2-matg[2,]);
+                    matG[1,] <- runif(nsim,-2/(lagsModelMax-1),2-matG[2,]);
                 }
             }
         }
     }
     else{
-        matg[,] <- rep(persistence,nsim);
+        matG[,] <- rep(persistence,nsim);
     }
 
-# Generate initial states of level and trend if they were not supplied
+    # Generate initial states of level and trend if they were not supplied
     if(is.null(initial)){
         if(Ttype=="N"){
-            arrvt[1:lagsModelMax,1,] <- runif(nsim,0,1000);
+            arrVt[1,1:lagsModelMax,] <- rep(runif(nsim,0,1000), each=lagsModelMax);
         }
         else if(Ttype=="A"){
-            arrvt[1:lagsModelMax,1,] <- runif(nsim,0,5000);
-            arrvt[1:lagsModelMax,2,] <- runif(nsim,-100,100);
+            arrVt[1,1:lagsModelMax,] <- rep(runif(nsim,0,5000), each=lagsModelMax);
+            arrVt[2,1:lagsModelMax,] <- rep(runif(nsim,-100,100), each=lagsModelMax);
         }
         else{
-            arrvt[1:lagsModelMax,1,] <- runif(nsim,500,5000);
-            arrvt[1:lagsModelMax,2,] <- 1;
+            arrVt[1,1:lagsModelMax,] <- rep(runif(nsim,500,5000), each=lagsModelMax);
+            arrVt[2,1:lagsModelMax,] <- 1;
         }
-        initial <- matrix(arrvt[1,1:componentsNumber,],ncol=nsim);
     }
     else{
-        arrvt[,1:componentsNumber,] <- rep(rep(initial,each=(obs+lagsModelMax)),nsim);
-        initial <- matrix(arrvt[1,1:componentsNumber,],ncol=nsim);
+        arrVt[1:componentsNumber,1:lagsModelMax,] <- rep(rep(initial,lagsModelMax),nsim);
     }
+    initial <- matrix(arrVt[1:componentsNumber,1,],nrow=nsim);
 
-# Generate seasonal states if they were not supplied
+    # Generate seasonal states if they were not supplied
     if(componentSeasonal & is.null(initialSeason)){
-# Create and normalize seasonal components. Use geometric mean for multiplicative case
+        # Create and normalize seasonal components. Use geometric mean for multiplicative case
         if(Stype == "A"){
-            arrvt[1:lagsModelMax,componentsNumber+1,] <- runif(nsim*lagsModelMax,-500,500);
+            arrVt[componentsNumber+1,1:lagsModelMax,] <- runif(nsim*lagsModelMax,-500,500);
             for(i in 1:nsim){
-                arrvt[1:lagsModelMax,componentsNumber+1,i] <- arrvt[1:lagsModelMax,componentsNumber+1,i] - mean(arrvt[1:lagsModelMax,componentsNumber+1,i]);
+                arrVt[componentsNumber+1,1:lagsModelMax,i] <- arrVt[componentsNumber+1,1:lagsModelMax,i] -
+                    mean(arrVt[componentsNumber+1,1:lagsModelMax,i]);
             }
         }
         else{
-            arrvt[1:lagsModelMax,componentsNumber+1,] <- runif(nsim*lagsModelMax,0.3,1.7);
+            arrVt[componentsNumber+1,1:lagsModelMax,] <- runif(nsim*lagsModelMax,0.3,1.7);
             for(i in 1:nsim){
-                arrvt[1:lagsModelMax,componentsNumber+1,i] <- arrvt[1:lagsModelMax,componentsNumber+1,i] / exp(mean(log(arrvt[1:lagsModelMax,componentsNumber+1,i])));
+                arrVt[componentsNumber+1,1:lagsModelMax,i] <- arrVt[componentsNumber+1,1:lagsModelMax,i] /
+                    exp(mean(log(arrVt[componentsNumber+1,1:lagsModelMax,i])));
             }
         }
-        initialSeason <- matrix(arrvt[1:lagsModelMax,componentsNumber+1,],ncol=nsim);
+        initialSeason <- matrix(arrVt[componentsNumber+1,1:lagsModelMax,],nrow=nsim);
+        # Count seasonal as a component
+        componentsNumber[] <- componentsNumber+1;
     }
-# If the seasonal model is chosen, fill in the first "frequency" values of seasonal component.
+    # If the seasonal model is chosen, fill in the first "frequency" values of seasonal component.
     else if(componentSeasonal & !is.null(initialSeason)){
-        arrvt[1:lagsModelMax,componentsNumber+1,] <- rep(initialSeason,nsim);
-        initialSeason <- matrix(arrvt[1:lagsModelMax,componentsNumber+1,],ncol=nsim);
+        arrVt[componentsNumber+1,1:lagsModelMax,] <- rep(initialSeason,nsim);
+        initialSeason <- matrix(arrVt[componentsNumber+1,1:lagsModelMax,],nrow=nsim);
+        # Count seasonal as a component
+        componentsNumber[] <- componentsNumber+1;
     }
 
-# Check if any argument was passed in dots
+    # Check if any argument was passed in dots
     if(length(ellipsis)==0){
         ellipsis$n <- nsim*obs;
-# Create vector of the errors
+        # Create vector of the errors
         if(any(randomizer==c("rnorm","rlaplace","rs"))){
-            materrors[,] <- do.call(randomizer,ellipsis);
+            matErrors[,] <- do.call(randomizer,ellipsis);
         }
         else if(randomizer=="rt"){
             # The degrees of freedom are df = n - k.
-            materrors[,] <- rt(nsim*obs,obs-(persistenceLength + lagsModelMax));
+            matErrors[,] <- rt(nsim*obs,obs-(persistenceLength + lagsModelMax));
         }
         else if(randomizer=="rlnorm"){
-            materrors[,] <- rlnorm(n=nsim*obs,0,0.01+(1-probability));
-            materrors <- materrors - 1;
+            matErrors[,] <- rlnorm(n=nsim*obs,0,0.01+(1-probability));
+            matErrors <- matErrors - 1;
         }
 
         if(randomizer!="rlnorm"){
@@ -475,83 +473,105 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
             if(Etype=="M"){
                 # Errors will be lognormal, decrease variance, so it behaves better
                 if(any(probability!=1)){
-                    materrors <- materrors * 0.5;
+                    matErrors <- matErrors * 0.5;
                 }
                 else{
-                    materrors <- materrors * 0.1;
+                    matErrors <- matErrors * 0.1;
                 }
-                materrors <- exp(materrors) - 1;
-#            exceedingerrors <- apply(abs(materrors),2,max)>1;
-#            materrors[,exceedingerrors] <- 0.95 * materrors[,exceedingerrors] / apply(abs(matrix(materrors[,exceedingerrors],obs)),2,max);
+                matErrors <- exp(matErrors) - 1;
             }
             else if(Etype=="A"){
-# Change variance to make some sense. Errors should not be rediculously high and not too low.
-                if(all(arrvt[1,1,]!=0)){
-                    materrors <- materrors * sqrt(abs(arrvt[1,1,]));
+                # Change variance to make some sense. Errors should not be rediculously high and not too low.
+                if(all(arrVt[1,1,]!=0)){
+                    matErrors <- matErrors * sqrt(abs(arrVt[1,1,]));
                 }
 
                 if(randomizer=="rs"){
-                    materrors <- materrors / 4;
+                    matErrors <- matErrors / 4;
                 }
             }
         }
     }
-# If arguments are passed, use them. WE ASSUME HERE THAT USER KNOWS WHAT HE'S DOING!
+    # If arguments are passed, use them. WE ASSUME HERE THAT USER KNOWS WHAT HE'S DOING!
     else{
         ellipsis$n <- nsim*obs;
-        materrors[,] <- do.call(randomizer,ellipsis);
+        matErrors[,] <- do.call(randomizer,ellipsis);
         if(randomizer=="rbeta"){
-# Center the errors around 0
-            materrors <- materrors - 0.5;
-# Make a meaningful variance of data. Something resembling to var=1.
-            materrors <- materrors / rep(sqrt(colMeans(materrors^2)) * sqrt(abs(arrvt[1,1,])),each=obs);
+            # Center the errors around 0
+            matErrors <- matErrors - 0.5;
+            # Make a meaningful variance of data. Something resembling to var=1.
+            matErrors <- matErrors / rep(sqrt(colMeans(matErrors^2)) * sqrt(abs(arrVt[1,1,])),each=obs);
         }
         else if(randomizer=="rt"){
-# Make a meaningful variance of data.
-            materrors <- materrors * rep(sqrt(abs(arrvt[1,1,])),each=obs);
+            # Make a meaningful variance of data.
+            matErrors <- matErrors * rep(sqrt(abs(arrVt[1,1,])),each=obs);
         }
 
         # Substitute 1 to get epsilon_t
         if(Etype=="M"){
-            materrors <- materrors - 1;
+            matErrors <- matErrors - 1;
         }
     }
 
-# Generate ones for the possible intermittency
+    # Generate ones for the possible intermittency
     if(all(probability == 1)){
-        matot[,] <- 1;
+        matOt[,] <- 1;
     }
     else{
-        matot[,] <- rbinom(obs*nsim,1,probability);
+        matOt[,] <- rbinom(obs*nsim,1,probability);
     }
 
-#### Simulate the data ####
-    simulateddata <- simulatorwrap(arrvt,materrors,matot,arrF,matw,matg,Etype,Ttype,Stype,lagsModel);
+    #### Variables for the adamCore ####
+    xregNumber <- 0;
+    constantRequired <- FALSE;
+    adamETS <- FALSE;
+    # Create all the necessary matrices and vectors
+    componentsNumberARIMA <- 0;
+    componentsNumberETS <- componentsNumber;
 
-    # if(all(probability == 1)){
-        matyt <- simulateddata$matyt;
-    # }
-    # else{
-        # matyt <- round(simulateddata$matyt,0);
-    # }
-    arrvt <- simulateddata$arrvt;
-    dimnames(arrvt) <- list(NULL,componentsNames,NULL);
+    componentsNumberETSSeasonal <- c(0,1)[componentSeasonal+1];
+    componentsNumberETSNonSeasonal <- componentsNumberETS - componentsNumberETSSeasonal;
+
+    profiles <- adamProfileCreator(lagsModel, lagsModelMax, obs);
+    indexLookupTable <- profiles$lookup;
+    profilesRecentArray <- arrVt[,1:lagsModelMax,, drop=FALSE];
+
+    # Create C++ adam class, which will then use fit, forecast etc methods
+    adamCpp <- new(adamCore,
+                   lagsModel, Etype, Ttype, Stype,
+                   componentsNumberETSNonSeasonal,
+                   componentsNumberETSSeasonal,
+                   componentsNumberETS, componentsNumberARIMA,
+                   xregNumber, length(lagsModel),
+                   constantRequired, adamETS);
+
+    #### Simulate the data ####
+    simulateddata <- adamCpp$simulate(matErrors, matOt,
+                                      arrVt, matWt,
+                                      arrF,
+                                      matG,
+                                      indexLookupTable, profilesRecentArray,
+                                      Etype);
+
+    matYt <- simulateddata$data;
+    arrVt[] <- simulateddata$states;
+    profilesRecentArray[] <- simulateddata$profile;
 
     if(any(randomizer==c("rnorm","rt"))){
-        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(materrors^2)));
+        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(matErrors^2)));
     }
     else if(randomizer=="rlaplace"){
-        veclikelihood <- -obs*(log(2*exp(1)) + log(colMeans(abs(materrors))));
+        veclikelihood <- -obs*(log(2*exp(1)) + log(colMeans(abs(matErrors))));
     }
     else if(randomizer=="rs"){
-        veclikelihood <- -2*obs*(log(2*exp(1)) + log(0.5*colMeans(sqrt(abs(materrors)))));
+        veclikelihood <- -2*obs*(log(2*exp(1)) + log(0.5*colMeans(sqrt(abs(matErrors)))));
     }
     else if(randomizer=="rlnorm"){
-        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(materrors^2))) - colSums(log(matyt));
+        veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(matErrors^2))) - colSums(log(matYt));
     }
     else if(randomizer=="rinvgauss"){
-        veclikelihood <- -0.5*(obs*(log(colMeans(materrors^2/(1+materrors))/(2*pi))-1) +
-                                   sum(log(matyt/(1+materrors))) - 3*sum(log(matyt)));
+        veclikelihood <- -0.5*(obs*(log(colMeans(matErrors^2/(1+matErrors))/(2*pi))-1) +
+                                   sum(log(matYt/(1+matErrors))) - 3*sum(log(matYt)));
     }
     # If this is something unknown, forget about it
     else{
@@ -559,22 +579,22 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
     }
 
     if(nsim==1){
-        matyt <- ts(matyt[,1],frequency=frequency);
-        materrors <- ts(materrors[,1],frequency=frequency);
-        arrvt <- ts(arrvt[,,1],frequency=frequency,start=c(0,frequency-lagsModelMax+1));
-        matot <- ts(matot[,1],frequency=frequency);
+        matYt <- ts(matYt[,1],frequency=frequency);
+        matErrors <- ts(matErrors[,1],frequency=frequency);
+        arrVt <- ts(arrVt[,,1],frequency=frequency,start=c(0,frequency-lagsModelMax+1));
+        matOt <- ts(matOt[,1],frequency=frequency);
     }
     else{
-        matyt <- ts(matyt,frequency=frequency);
-        materrors <- ts(materrors,frequency=frequency);
-        matot <- ts(matot,frequency=frequency);
+        matYt <- ts(matYt,frequency=frequency);
+        matErrors <- ts(matErrors,frequency=frequency);
+        matOt <- ts(matOt,frequency=frequency);
     }
 
     if(Ttype!="N"){
-        rownames(matg) <- c("alpha","beta","gamma")[1:persistenceLength];
+        rownames(matG) <- c("alpha","beta","gamma")[1:persistenceLength];
     }
     else{
-        rownames(matg) <- c("alpha","gamma")[1:persistenceLength];
+        rownames(matG) <- c("alpha","gamma")[1:persistenceLength];
     }
 
     model <- paste0("ETS(",model,")");
@@ -582,12 +602,14 @@ sim.es <- function(model="ANN", obs=10, nsim=1,
         model <- paste0("i",model);
     }
 
-    if(any(is.nan(matyt))){
+    if(any(is.nan(matYt))){
         warning("NaN values were produced by the simulator.",call.=FALSE);
     }
 
-    model <- list(model=model, data=matyt, states=arrvt, persistence=matg, phi=phi,
-                  initial=initial, initialSeason=initialSeason, probability=probability, intermittent=intermittent,
-                  residuals=materrors, occurrence=matot, logLik=veclikelihood, other=ellipsis);
+    model <- list(model=model, data=matYt, states=arrVt, persistence=matG, phi=phi,
+                  initial=initial, initialSeason=initialSeason,
+                  profile=profilesRecentArray,
+                  probability=probability, intermittent=intermittent,
+                  residuals=matErrors, occurrence=matOt, logLik=veclikelihood, other=ellipsis);
     return(structure(model,class="smooth.sim"));
 }
